# encoding: utf-8
"""
testrunner.py

Created by James Casey on 2009-04-30.
Copyright (c) 2009 CERN. All rights reserved.
"""

import os, sys, getopt, logging
import signal  
log = logging.getLogger(__name__)

from swatc.common import config, logger, event

def _make_instance(klass, args):
    try:
        mod_bits = klass.split('.')
        klassname = mod_bits[-1]
        modulename = '.'.join(mod_bits[:-1])
        __import__(modulename) 
        module=sys.modules[modulename] 
        return getattr(module,klassname)(**args)
    except Exception, err:
        log.error("%s could not be initialized: %s" % (str(klass),str(err)))
        log.exception(err)
    return None


class TestManager:
    """Manages a producer- and consumer-classes with an event."""
    
    def __init__(self):
        """Init Testmanager."""
        self.producer = []
        self.consumer = []
        self.all = []
        self.event = event.Event()
        
    def addProducer(self, klass, args):
        """Adds a producer to the TestManager.
        
        Each Producer's run method will be called with the global event.
        klass: definies the path to the python class.
        args: definies a dictionary of arguments for the Producers init method.
    """
        inst = _make_instance(klass, args)
        if inst:
            self.producer.append(inst)
            self.all.append(inst)
        
    def addConsumer(self, klass, args):
        """Adds a consumer to the TestManager.
        
        Each Consumer's consume method will be called, when a Producer triggers
        the global event.
        MODULE definies the path to the python class.
        ARGS definies a dictionary of arguments for the Producers init method.
        """
        inst = _make_instance(klass, args)
        if inst:
            self.consumer.append(inst)
            self.all.append(inst)
            self.event.append(inst.consume)
    
    def doSetup(self):
        """Setup all modules by calling their setup methods."""
        for c in self.all:
            try:
                log.debug("setup %s" % str(c))
                c.setup()
            except Exception, err:
                log.warning("%s setup failed: %s" % (str(c),str(err)))
                log.exception(err)
    
    def doRun(self):
        """Run all producers by calling their run methods."""
        for c in self.producer:
            try:
                log.debug("run %s" % str(c))
                c.run(self.event)
            except Exception, err:
                log.warning("%s failed while running: %s" % (str(c),str(err)))
                log.exception(err)      
    
    def doCleanup(self):
        """Cleanup all modules by calling their cleanup modules.
        
        This happens in the reverse order of doSetup().
        """
        reverse = []
        for a in self.all:
            reverse.insert(0,a)
        for c in reverse:
            try:
                log.debug("clean up %s" % str(c))
                c.cleanup()
            except Exception, err:
                log.warning("%s cleanup failed: %s" % (str(c),str(err)))
                log.exception(err)
